@@ -12,6 +12,7 @@ from narwhals._pandas_like.utils import (
     align_and_extract_native,
     get_dtype_backend,
     import_array_module,
+    iter_dtype_backends,
     native_to_narwhals_dtype,
     object_native_to_narwhals_dtype,
     rename,
@@ -27,9 +28,8 @@ from narwhals._utils import (
     exclude_column_names,
     generate_temporary_column_name,
     parse_columns_to_drop,
-    parse_version,
     scale_bytes,
-    validate_backend_version,
+    zip_strict,
 )
 from narwhals.dependencies import is_pandas_like_dataframe
 from narwhals.exceptions import InvalidOperationError, ShapeError
@@ -47,13 +47,15 @@ if TYPE_CHECKING:
     from narwhals._pandas_like.expr import PandasLikeExpr
     from narwhals._pandas_like.group_by import PandasLikeGroupBy
     from narwhals._pandas_like.namespace import PandasLikeNamespace
+    from narwhals._spark_like.utils import SparkSession
     from narwhals._translate import IntoArrowTable
-    from narwhals._utils import Version, _FullContext
+    from narwhals._typing import _EagerAllowedImpl, _LazyAllowedImpl
+    from narwhals._utils import Version, _LimitedContext
     from narwhals.dtypes import DType
-    from narwhals.schema import Schema
     from narwhals.typing import (
         AsofJoinStrategy,
         DTypeBackend,
+        IntoSchema,
         JoinStrategy,
         PivotAgg,
         SizedMultiIndexSelector,
@@ -102,29 +104,34 @@ class PandasLikeDataFrame(
         native_dataframe: Any,
         *,
         implementation: Implementation,
-        backend_version: tuple[int, ...],
         version: Version,
         validate_column_names: bool,
+        validate_backend_version: bool = False,
     ) -> None:
         self._native_frame = native_dataframe
         self._implementation = implementation
-        self._backend_version = backend_version
         self._version = version
-        validate_backend_version(self._implementation, self._backend_version)
         if validate_column_names:
             check_column_names_are_unique(native_dataframe.columns)
+        if validate_backend_version:
+            self._validate_backend_version()
 
     @classmethod
-    def from_arrow(cls, data: IntoArrowTable, /, *, context: _FullContext) -> Self:
+    def from_arrow(cls, data: IntoArrowTable, /, *, context: _LimitedContext) -> Self:
         implementation = context._implementation
         tbl = _into_arrow_table(data, context)
         if implementation.is_pandas():
             native = tbl.to_pandas()
-        elif implementation.is_modin():  # pragma: no cover
-            from modin.pandas.utils import (
-                from_arrow as mpd_from_arrow,  # pyright: ignore[reportAttributeAccessIssue]
-            )
-
+        elif implementation.is_modin():
+            # NOTE: Function moved + deprecated (0.26.0), then old path removed (0.31.0)
+            # https://github.com/modin-project/modin/pull/6806
+            # https://github.com/modin-project/modin/pull/7274
+            if implementation._backend_version() >= (0, 26, 0):
+                from modin.pandas.io import from_arrow as mpd_from_arrow
+            else:  # pragma: no cover
+                from modin.pandas.utils import (
+                    from_arrow as mpd_from_arrow,  # pyright: ignore[reportAttributeAccessIssue]
+                )
             native = mpd_from_arrow(tbl)
         elif implementation.is_cudf():  # pragma: no cover
             native = implementation.to_native_namespace().DataFrame.from_arrow(tbl)
@@ -139,15 +146,15 @@ class PandasLikeDataFrame(
         data: Mapping[str, Any],
         /,
         *,
-        context: _FullContext,
-        schema: Mapping[str, DType] | Schema | None,
+        context: _LimitedContext,
+        schema: IntoSchema | None,
     ) -> Self:
         from narwhals.schema import Schema
 
         implementation = context._implementation
         ns = implementation.to_native_namespace()
-        Series = cast("type[pd.Series[Any]]", ns.Series)  # noqa: N806
-        DataFrame = cast("type[pd.DataFrame]", ns.DataFrame)  # noqa: N806
+        Series = cast("type[pd.Series[Any]]", ns.Series)
+        DataFrame = cast("type[pd.DataFrame]", ns.DataFrame)
         aligned_data: dict[str, pd.Series[Any] | Any] = {}
         left_most: PandasLikeSeries | None = None
         for name, series in data.items():
@@ -160,13 +167,40 @@ class PandasLikeDataFrame(
                     aligned_data[name] = align_and_extract_native(left_most, compliant)[1]
             else:
                 aligned_data[name] = series
-
-        native = DataFrame.from_dict(aligned_data)
+        if aligned_data or not schema:
+            native = DataFrame.from_dict(aligned_data)
+        else:
+            native = DataFrame.from_dict({col: [] for col in schema})
         if schema:
-            it: Iterable[DTypeBackend] = (
-                get_dtype_backend(dtype, implementation) for dtype in native.dtypes
-            )
-            native = native.astype(Schema(schema).to_pandas(it))
+            backend: Iterable[DTypeBackend] | None = None
+            if aligned_data:
+                backend = iter_dtype_backends(native.dtypes, implementation)
+            native = native.astype(Schema(schema).to_pandas(backend))
+        return cls.from_native(native, context=context)
+
+    @classmethod
+    def from_dicts(
+        cls,
+        data: Sequence[Mapping[str, Any]],
+        /,
+        *,
+        context: _LimitedContext,
+        schema: IntoSchema | None,
+    ) -> Self:
+        from narwhals.schema import Schema
+
+        implementation = context._implementation
+        ns = implementation.to_native_namespace()
+        DataFrame = cast("type[pd.DataFrame]", ns.DataFrame)
+        if data or not schema:
+            native = DataFrame.from_records(data)
+        else:
+            native = DataFrame.from_dict({col: [] for col in schema})
+        if schema:
+            backend: Iterable[DTypeBackend] | None = None
+            if data:
+                backend = iter_dtype_backends(native.dtypes, implementation)
+            native = native.astype(Schema(schema).to_pandas(backend))
         return cls.from_native(native, context=context)
 
     @staticmethod
@@ -174,11 +208,10 @@ class PandasLikeDataFrame(
         return is_pandas_like_dataframe(obj)  # pragma: no cover
 
     @classmethod
-    def from_native(cls, data: Any, /, *, context: _FullContext) -> Self:
+    def from_native(cls, data: Any, /, *, context: _LimitedContext) -> Self:
         return cls(
             data,
             implementation=context._implementation,
-            backend_version=context._backend_version,
             version=context._version,
             validate_column_names=True,
         )
@@ -189,13 +222,13 @@ class PandasLikeDataFrame(
         data: _2DArray,
         /,
         *,
-        context: _FullContext,
-        schema: Mapping[str, DType] | Schema | Sequence[str] | None,
+        context: _LimitedContext,
+        schema: IntoSchema | Sequence[str] | None,
     ) -> Self:
         from narwhals.schema import Schema
 
         implementation = context._implementation
-        DataFrame: Constructor = implementation.to_native_namespace().DataFrame  # noqa: N806
+        DataFrame: Constructor = implementation.to_native_namespace().DataFrame
         if isinstance(schema, (Mapping, Schema)):
             it: Iterable[DTypeBackend] = (
                 get_dtype_backend(native_type, implementation)
@@ -217,9 +250,7 @@ class PandasLikeDataFrame(
     def __narwhals_namespace__(self) -> PandasLikeNamespace:
         from narwhals._pandas_like.namespace import PandasLikeNamespace
 
-        return PandasLikeNamespace(
-            self._implementation, self._backend_version, version=self._version
-        )
+        return PandasLikeNamespace(self._implementation, version=self._version)
 
     def __native_namespace__(self) -> ModuleType:
         if self._implementation in {
@@ -239,7 +270,6 @@ class PandasLikeDataFrame(
         return self.__class__(
             self.native,
             implementation=self._implementation,
-            backend_version=self._backend_version,
             version=version,
             validate_column_names=False,
         )
@@ -248,7 +278,6 @@ class PandasLikeDataFrame(
         return self.__class__(
             df,
             implementation=self._implementation,
-            backend_version=self._backend_version,
             version=self._version,
             validate_column_names=validate_column_names,
         )
@@ -262,12 +291,7 @@ class PandasLikeDataFrame(
             msg = f"Expected object of length {len_idx}, got: {len_other}."
             raise ShapeError(msg)
         if other.native.index is not index:
-            return set_index(
-                other.native,
-                index,
-                implementation=other._implementation,
-                backend_version=other._backend_version,
-            )
+            return set_index(other.native, index, implementation=other._implementation)
         return other.native
 
     @property
@@ -276,8 +300,7 @@ class PandasLikeDataFrame(
             import numpy as np
 
             return np
-        else:
-            return import_array_module(self._implementation)
+        return import_array_module(self._implementation)
 
     def get_column(self, name: str) -> PandasLikeSeries:
         return PandasLikeSeries.from_native(self.native[name], context=self)
@@ -390,12 +413,7 @@ class PandasLikeDataFrame(
     # --- reshape ---
     def simple_select(self, *column_names: str) -> Self:
         return self._with_native(
-            select_columns_by_name(
-                self.native,
-                list(column_names),
-                self._backend_version,
-                self._implementation,
-            ),
+            select_columns_by_name(self.native, list(column_names), self._implementation),
             validate_column_names=False,
         )
 
@@ -476,12 +494,7 @@ class PandasLikeDataFrame(
 
     def rename(self, mapping: Mapping[str, str]) -> Self:
         return self._with_native(
-            rename(
-                self.native,
-                columns=mapping,
-                implementation=self._implementation,
-                backend_version=self._backend_version,
-            )
+            rename(self.native, columns=mapping, implementation=self._implementation)
         )
 
     def drop(self, columns: Sequence[str], *, strict: bool) -> Self:
@@ -503,51 +516,55 @@ class PandasLikeDataFrame(
             validate_column_names=False,
         )
 
+    def top_k(self, k: int, *, by: Iterable[str], reverse: bool | Sequence[bool]) -> Self:
+        df = self.native
+        schema = self.schema
+        if isinstance(reverse, bool) and all(schema[x].is_numeric() for x in by):
+            if reverse:
+                return self._with_native(df.nsmallest(k, by))
+            return self._with_native(df.nlargest(k, by))
+        return self._with_native(
+            df.sort_values(list(by), ascending=reverse).head(k),
+            validate_column_names=False,
+        )
+
     # --- convert ---
     def collect(
-        self, backend: Implementation | None, **kwargs: Any
+        self, backend: _EagerAllowedImpl | None, **kwargs: Any
     ) -> CompliantDataFrameAny:
         if backend is None:
             return PandasLikeDataFrame(
                 self.native,
                 implementation=self._implementation,
-                backend_version=self._backend_version,
                 version=self._version,
                 validate_column_names=False,
             )
 
         if backend is Implementation.PANDAS:
-            import pandas as pd  # ignore-banned-import
-
-            return PandasLikeDataFrame(
-                self.to_pandas(),
-                implementation=Implementation.PANDAS,
-                backend_version=parse_version(pd),
-                version=self._version,
-                validate_column_names=False,
-            )
+            kwds: dict[str, Any] = {
+                "implementation": Implementation.PANDAS,
+                "version": self._version,
+                "validate_column_names": False,
+            }
+            if backend is not self._implementation:
+                kwds.update(validate_backend_version=True)
+            return PandasLikeDataFrame(self.to_pandas(), **kwds)
 
         if backend is Implementation.PYARROW:
-            import pyarrow as pa  # ignore-banned-import
-
             from narwhals._arrow.dataframe import ArrowDataFrame
 
             return ArrowDataFrame(
                 native_dataframe=self.to_arrow(),
-                backend_version=parse_version(pa),
+                validate_backend_version=True,
                 version=self._version,
                 validate_column_names=False,
             )
 
         if backend is Implementation.POLARS:
-            import polars as pl  # ignore-banned-import
-
             from narwhals._polars.dataframe import PolarsDataFrame
 
             return PolarsDataFrame(
-                df=self.to_polars(),
-                backend_version=parse_version(pl),
-                version=self._version,
+                df=self.to_polars(), validate_backend_version=True, version=self._version
             )
 
         msg = f"Unsupported `backend` value: {backend}"  # pragma: no cover
@@ -584,10 +601,13 @@ class PandasLikeDataFrame(
         )
         extra = [
             right_key if right_key not in self.columns else f"{right_key}{suffix}"
-            for left_key, right_key in zip(left_on, right_on)
+            for left_key, right_key in zip_strict(left_on, right_on)
             if right_key != left_key
         ]
-        return result_native.drop(columns=extra)
+        # NOTE: Keep `inplace=True` to avoid making a redundant copy.
+        # This may need updating, depending on https://github.com/pandas-dev/pandas/pull/51466/files
+        result_native.drop(columns=extra, inplace=True)  # noqa: PD002
+        return result_native
 
     def _join_full(
         self, other: Self, *, left_on: Sequence[str], right_on: Sequence[str], suffix: str
@@ -614,17 +634,17 @@ class PandasLikeDataFrame(
             key_token = generate_temporary_column_name(
                 n_bytes=8, columns=(*self.columns, *other.columns)
             )
-            return (
-                self.native.assign(**{key_token: 0})
-                .merge(
-                    other.native.assign(**{key_token: 0}),
-                    how="inner",
-                    left_on=key_token,
-                    right_on=key_token,
-                    suffixes=("", suffix),
-                )
-                .drop(columns=key_token)
+            result_native = self.native.assign(**{key_token: 0}).merge(
+                other.native.assign(**{key_token: 0}),
+                how="inner",
+                left_on=key_token,
+                right_on=key_token,
+                suffixes=("", suffix),
             )
+            # NOTE: Keep `inplace=True` to avoid making a redundant copy.
+            # This may need updating, depending on https://github.com/pandas-dev/pandas/pull/51466/files
+            result_native.drop(columns=key_token, inplace=True)  # noqa: PD002
+            return result_native
         return self.native.merge(other.native, how="cross", suffixes=("", suffix))
 
     def _join_semi(
@@ -658,18 +678,18 @@ class PandasLikeDataFrame(
             columns_to_select=list(right_on),
             columns_mapping=dict(zip(right_on, left_on)),
         )
-        return (
-            self.native.merge(
-                other_native,
-                # TODO(FBruzzesi): See https://github.com/modin-project/modin/issues/7384
-                how="left" if implementation.is_pandas() else "outer",
-                indicator=indicator_token,
-                left_on=left_on,
-                right_on=left_on,
-            )
-            .loc[lambda t: t[indicator_token] == "left_only"]
-            .drop(columns=indicator_token)
-        )
+        result_native = self.native.merge(
+            other_native,
+            # TODO(FBruzzesi): See https://github.com/modin-project/modin/issues/7384
+            how="left" if implementation.is_pandas() else "outer",
+            indicator=indicator_token,
+            left_on=left_on,
+            right_on=left_on,
+        ).loc[lambda t: t[indicator_token] == "left_only"]
+        # NOTE: Keep `inplace=True` to avoid making a redundant copy.
+        # This may need updating, depending on https://github.com/pandas-dev/pandas/pull/51466/files
+        result_native.drop(columns=indicator_token, inplace=True)  # noqa: PD002
+        return result_native
 
     def _join_filter_rename(
         self, other: Self, columns_to_select: list[str], columns_mapping: dict[str, str]
@@ -681,18 +701,14 @@ class PandasLikeDataFrame(
         Notice that a native object is returned.
         """
         implementation = self._implementation
-        backend_version = self._backend_version
-
         return rename(
             select_columns_by_name(
                 other.native,
                 column_names=columns_to_select,
-                backend_version=backend_version,
                 implementation=implementation,
             ),
             columns=columns_mapping,
             implementation=implementation,
-            backend_version=backend_version,
         ).drop_duplicates()
 
     def join(
@@ -770,55 +786,95 @@ class PandasLikeDataFrame(
         *,
         keep: UniqueKeepStrategy,
         maintain_order: bool | None = None,
+        order_by: Sequence[str] | None,
     ) -> Self:
         # The param `maintain_order` is only here for compatibility with the Polars API
         # and has no effect on the output.
         mapped_keep = {"none": False, "any": "first"}.get(keep, keep)
         if subset and (error := self._check_columns_exist(subset)):
             raise error
-        return self._with_native(
-            self.native.drop_duplicates(subset=subset, keep=mapped_keep),
-            validate_column_names=False,
-        )
+        if order_by and maintain_order:
+            token = generate_temporary_column_name(8, self.columns)
+            res = (
+                self.with_row_index(token, order_by=None)
+                .sort(*order_by, nulls_last=False, descending=False)
+                .native.drop_duplicates(subset or self.columns, keep=mapped_keep)
+                .sort_values(token)
+            )
+            res.drop(columns=token, inplace=True)  # noqa: PD002
+        elif order_by:
+            res = self.sort(
+                *order_by, nulls_last=False, descending=False
+            ).native.drop_duplicates(subset or self.columns, keep=mapped_keep)
+        else:
+            res = self.native.drop_duplicates(subset or self.columns, keep=mapped_keep)
+        return self._with_native(res, validate_column_names=False)
 
     # --- lazy-only ---
-    def lazy(self, *, backend: Implementation | None = None) -> CompliantLazyFrameAny:
-        from narwhals.utils import parse_version
-
+    def lazy(
+        self,
+        backend: _LazyAllowedImpl | None = None,
+        *,
+        session: SparkSession | None = None,
+    ) -> CompliantLazyFrameAny:
         pandas_df = self.to_pandas()
         if backend is None:
             return self
-        elif backend is Implementation.DUCKDB:
+        if backend is Implementation.DUCKDB:
             import duckdb  # ignore-banned-import
 
             from narwhals._duckdb.dataframe import DuckDBLazyFrame
 
             return DuckDBLazyFrame(
                 df=duckdb.table("pandas_df"),
-                backend_version=parse_version(duckdb),
+                validate_backend_version=True,
                 version=self._version,
             )
-        elif backend is Implementation.POLARS:
+        if backend is Implementation.POLARS:
             import polars as pl  # ignore-banned-import
 
             from narwhals._polars.dataframe import PolarsLazyFrame
 
             return PolarsLazyFrame(
                 df=pl.from_pandas(pandas_df).lazy(),
-                backend_version=parse_version(pl),
+                validate_backend_version=True,
                 version=self._version,
             )
-        elif backend is Implementation.DASK:
-            import dask  # ignore-banned-import
+        if backend is Implementation.DASK:
             import dask.dataframe as dd  # ignore-banned-import
 
             from narwhals._dask.dataframe import DaskLazyFrame
 
             return DaskLazyFrame(
                 native_dataframe=dd.from_pandas(pandas_df),
-                backend_version=parse_version(dask),
+                validate_backend_version=True,
                 version=self._version,
             )
+        if backend is Implementation.IBIS:
+            import ibis  # ignore-banned-import
+
+            from narwhals._ibis.dataframe import IbisLazyFrame
+
+            return IbisLazyFrame(
+                ibis.memtable(pandas_df, columns=self.columns),
+                validate_backend_version=True,
+                version=self._version,
+            )
+
+        if backend.is_spark_like():
+            from narwhals._spark_like.dataframe import SparkLikeLazyFrame
+
+            if session is None:
+                msg = "Spark like backends require `session` to be not None."
+                raise ValueError(msg)
+
+            return SparkLikeLazyFrame(
+                session.createDataFrame(pandas_df),
+                version=self._version,
+                implementation=backend,
+                validate_backend_version=True,
+            )
+
         raise AssertionError  # pragma: no cover
 
     @property
@@ -883,9 +939,9 @@ class PandasLikeDataFrame(
     def to_pandas(self) -> pd.DataFrame:
         if self._implementation is Implementation.PANDAS:
             return self.native
-        elif self._implementation is Implementation.CUDF:
+        if self._implementation is Implementation.CUDF:
             return self.native.to_pandas()
-        elif self._implementation is Implementation.MODIN:
+        if self._implementation is Implementation.MODIN:
             return self.native._to_pandas()
         msg = f"Unknown implementation: {self._implementation}"  # pragma: no cover
         raise AssertionError(msg)
@@ -924,7 +980,7 @@ class PandasLikeDataFrame(
                 raise ValueError(msg)
             return self.native.iloc[0, 0]
 
-        elif row is None or column is None:
+        if row is None or column is None:
             msg = "cannot call `.item()` with only one of `row` or `column`"
             raise ValueError(msg)
 
@@ -1005,17 +1061,9 @@ class PandasLikeDataFrame(
         ],
         /,
     ) -> Any:
-        categorical = self._version.dtypes.Categorical
-        kwds: dict[Any, Any] = {"observed": True}
-        if self._implementation is Implementation.CUDF:
-            kwds.pop("observed")
-            cols = set(chain(values, index, on))
-            schema = self.schema.items()
-            if any(
-                tp for name, tp in schema if name in cols and isinstance(tp, categorical)
-            ):
-                msg = "`pivot` with Categoricals is not implemented for cuDF backend"
-                raise NotImplementedError(msg)
+        kwds: dict[Any, Any] = (
+            {} if self._implementation is Implementation.CUDF else {"observed": True}
+        )
         return self.native.pivot_table(
             values=values,
             index=index,
@@ -1035,7 +1083,7 @@ class PandasLikeDataFrame(
     ) -> pd.DataFrame:
         if aggregate_function is None:
             return self.native.pivot(columns=on, index=index, values=values)
-        elif aggregate_function == "len":
+        if aggregate_function == "len":
             return (
                 self.native.groupby([*on, *index], as_index=False)
                 .agg(dict.fromkeys(values, "size"))
@@ -1140,29 +1188,26 @@ class PandasLikeDataFrame(
             return self._with_native(
                 self.native.explode(columns[0]), validate_column_names=False
             )
-        else:
-            native_frame = self.native
-            anchor_series = native_frame[columns[0]].list.len()
+        native_frame = self.native
+        anchor_series = native_frame[columns[0]].list.len()
 
-            if not all(
-                (native_frame[col_name].list.len() == anchor_series).all()
-                for col_name in columns[1:]
-            ):
-                msg = "exploded columns must have matching element counts"
-                raise ShapeError(msg)
+        if not all(
+            (native_frame[col_name].list.len() == anchor_series).all()
+            for col_name in columns[1:]
+        ):
+            msg = "exploded columns must have matching element counts"
+            raise ShapeError(msg)
 
-            original_columns = self.columns
-            other_columns = [c for c in original_columns if c not in columns]
+        original_columns = self.columns
+        other_columns = [c for c in original_columns if c not in columns]
 
-            exploded_frame = native_frame[[*other_columns, columns[0]]].explode(
-                columns[0]
-            )
-            exploded_series = [
-                native_frame[col_name].explode().to_frame() for col_name in columns[1:]
-            ]
+        exploded_frame = native_frame[[*other_columns, columns[0]]].explode(columns[0])
+        exploded_series = [
+            native_frame[col_name].explode().to_frame() for col_name in columns[1:]
+        ]
 
-            plx = self.__native_namespace__()
-            return self._with_native(
-                plx.concat([exploded_frame, *exploded_series], axis=1)[original_columns],
-                validate_column_names=False,
-            )
+        plx = self.__native_namespace__()
+        return self._with_native(
+            plx.concat([exploded_frame, *exploded_series], axis=1)[original_columns],
+            validate_column_names=False,
+        )
