@@ -6,6 +6,8 @@ from __future__ import annotations
 import sys
 from typing import TYPE_CHECKING, Any
 
+from narwhals._exceptions import issue_warning
+
 if TYPE_CHECKING:
     import cudf
     import dask.dataframe as dd
@@ -26,6 +28,7 @@ if TYPE_CHECKING:
         IntoDataFrameT,
         IntoLazyFrameT,
         IntoSeriesT,
+        PandasLikeDType,
         _1DArray,
         _1DArrayInt,
         _2DArray,
@@ -125,24 +128,24 @@ def get_sqlframe() -> Any:
     return sys.modules.get("sqlframe", None)
 
 
-def _raise_if_narwhals_df_or_lf(df: Any) -> None:
+def _warn_if_narwhals_df_or_lf(df: Any) -> None:
     if is_narwhals_dataframe(df) or is_narwhals_lazyframe(df):
         msg = (
             f"You passed a `{type(df)}` to `is_pandas_dataframe`.\n\n"
             "Hint: Instead of e.g. `is_pandas_dataframe(df)`, "
             "did you mean `is_pandas_dataframe(df.to_native())`?"
         )
-        raise TypeError(msg)
+        issue_warning(msg, UserWarning)
 
 
-def _raise_if_narwhals_series(ser: Any) -> None:
+def _warn_if_narwhals_series(ser: Any) -> None:
     if is_narwhals_series(ser):
         msg = (
             f"You passed a `{type(ser)}` to `is_pandas_series`.\n\n"
             "Hint: Instead of e.g. `is_pandas_series(ser)`, "
             "did you mean `is_pandas_series(ser.to_native())`?"
         )
-        raise TypeError(msg)
+        issue_warning(msg, UserWarning)
 
 
 def is_pandas_dataframe(df: Any) -> TypeIs[pd.DataFrame]:
@@ -151,7 +154,7 @@ def is_pandas_dataframe(df: Any) -> TypeIs[pd.DataFrame]:
     Warning:
         This method cannot be called on a Narwhals DataFrame/LazyFrame.
     """
-    _raise_if_narwhals_df_or_lf(df)
+    _warn_if_narwhals_df_or_lf(df)
     return ((pd := get_pandas()) is not None and isinstance(df, pd.DataFrame)) or any(
         (mod := sys.modules.get(module_name, None)) is not None
         and isinstance(df, mod.pandas.DataFrame)
@@ -165,7 +168,7 @@ def is_pandas_series(ser: Any) -> TypeIs[pd.Series[Any]]:
     Warning:
         This method cannot be called on Narwhals Series.
     """
-    _raise_if_narwhals_series(ser)
+    _warn_if_narwhals_series(ser)
     return ((pd := get_pandas()) is not None and isinstance(ser, pd.Series)) or any(
         (mod := sys.modules.get(module_name, None)) is not None
         and isinstance(ser, mod.pandas.Series)
@@ -188,7 +191,7 @@ def is_modin_dataframe(df: Any) -> TypeIs[mpd.DataFrame]:
     Warning:
         This method cannot be called on a Narwhals DataFrame/LazyFrame.
     """
-    _raise_if_narwhals_df_or_lf(df)
+    _warn_if_narwhals_df_or_lf(df)
     return (mpd := get_modin()) is not None and isinstance(df, mpd.DataFrame)
 
 
@@ -198,7 +201,7 @@ def is_modin_series(ser: Any) -> TypeIs[mpd.Series]:
     Warning:
         This method cannot be called on Narwhals Series.
     """
-    _raise_if_narwhals_series(ser)
+    _warn_if_narwhals_series(ser)
     return (mpd := get_modin()) is not None and isinstance(ser, mpd.Series)
 
 
@@ -213,7 +216,7 @@ def is_cudf_dataframe(df: Any) -> TypeIs[cudf.DataFrame]:
     Warning:
         This method cannot be called on a Narwhals DataFrame/LazyFrame.
     """
-    _raise_if_narwhals_df_or_lf(df)
+    _warn_if_narwhals_df_or_lf(df)
     return (cudf := get_cudf()) is not None and isinstance(df, cudf.DataFrame)
 
 
@@ -223,7 +226,7 @@ def is_cudf_series(ser: Any) -> TypeIs[cudf.Series[Any]]:
     Warning:
         This method cannot be called on Narwhals Series.
     """
-    _raise_if_narwhals_series(ser)
+    _warn_if_narwhals_series(ser)
     return (cudf := get_cudf()) is not None and isinstance(ser, cudf.Series)
 
 
@@ -248,7 +251,7 @@ def is_dask_dataframe(df: Any) -> TypeIs[dd.DataFrame]:
     Warning:
         This method cannot be called on a Narwhals DataFrame/LazyFrame.
     """
-    _raise_if_narwhals_df_or_lf(df)
+    _warn_if_narwhals_df_or_lf(df)
     return (dd := get_dask_dataframe()) is not None and isinstance(df, dd.DataFrame)
 
 
@@ -258,7 +261,7 @@ def is_duckdb_relation(df: Any) -> TypeIs[duckdb.DuckDBPyRelation]:
     Warning:
         This method cannot be called on Narwhals DataFrame/LazyFrame.
     """
-    _raise_if_narwhals_df_or_lf(df)
+    _warn_if_narwhals_df_or_lf(df)
     return (duckdb := get_duckdb()) is not None and isinstance(
         df, duckdb.DuckDBPyRelation
     )
@@ -270,7 +273,7 @@ def is_ibis_table(df: Any) -> TypeIs[ibis.Table]:
     Warning:
         This method cannot be called on Narwhals DataFrame/LazyFrame.
     """
-    _raise_if_narwhals_df_or_lf(df)
+    _warn_if_narwhals_df_or_lf(df)
     return (ibis := get_ibis()) is not None and isinstance(df, ibis.expr.types.Table)
 
 
@@ -280,7 +283,7 @@ def is_polars_dataframe(df: Any) -> TypeIs[pl.DataFrame]:
     Warning:
         This method cannot be called on a Narwhals DataFrame/LazyFrame.
     """
-    _raise_if_narwhals_df_or_lf(df)
+    _warn_if_narwhals_df_or_lf(df)
     return (pl := get_polars()) is not None and isinstance(df, pl.DataFrame)
 
 
@@ -290,7 +293,7 @@ def is_polars_lazyframe(df: Any) -> TypeIs[pl.LazyFrame]:
     Warning:
         This method cannot be called on Narwhals DataFrame/LazyFrame.
     """
-    _raise_if_narwhals_df_or_lf(df)
+    _warn_if_narwhals_df_or_lf(df)
     return (pl := get_polars()) is not None and isinstance(df, pl.LazyFrame)
 
 
@@ -300,17 +303,28 @@ def is_polars_series(ser: Any) -> TypeIs[pl.Series]:
     Warning:
         This method cannot be called on Narwhals Series.
     """
-    _raise_if_narwhals_series(ser)
+    _warn_if_narwhals_series(ser)
     return (pl := get_polars()) is not None and isinstance(ser, pl.Series)
 
 
+def is_polars_schema(obj: Any) -> TypeIs[pl.Schema]:
+    return (
+        bool(pl := get_polars()) and hasattr(pl, "Schema") and isinstance(obj, pl.Schema)
+    )
+
+
+# NOTE: For `pl.Schema` only instantiated dtypes are expected
+def is_polars_data_type(obj: Any) -> TypeIs[pl.DataType]:
+    return bool(pl := get_polars()) and isinstance(obj, pl.DataType)
+
+
 def is_pyarrow_chunked_array(ser: Any) -> TypeIs[pa.ChunkedArray[Any]]:
     """Check whether `ser` is a PyArrow ChunkedArray without importing PyArrow.
 
     Warning:
         This method cannot be called on Narwhals Series.
     """
-    _raise_if_narwhals_series(ser)
+    _warn_if_narwhals_series(ser)
     return (pa := get_pyarrow()) is not None and isinstance(ser, pa.ChunkedArray)
 
 
@@ -320,7 +334,7 @@ def is_pyarrow_table(df: Any) -> TypeIs[pa.Table]:
     Warning:
         This method cannot be called on Narwhals DataFrame/LazyFrame.
     """
-    _raise_if_narwhals_df_or_lf(df)
+    _warn_if_narwhals_df_or_lf(df)
     return (pa := get_pyarrow()) is not None and isinstance(df, pa.Table)
 
 
@@ -328,13 +342,21 @@ def is_pyarrow_scalar(obj: Any) -> TypeIs[pa.Scalar[Any]]:
     return (pa := get_pyarrow()) is not None and isinstance(obj, pa.Scalar)
 
 
+def is_pyarrow_schema(obj: Any) -> TypeIs[pa.Schema]:
+    return bool(pa := get_pyarrow()) and isinstance(obj, pa.Schema)
+
+
+def is_pyarrow_data_type(obj: Any) -> TypeIs[pa.DataType]:
+    return bool(pa := get_pyarrow()) and isinstance(obj, pa.DataType)
+
+
 def is_pyspark_dataframe(df: Any) -> TypeIs[pyspark_sql.DataFrame]:
     """Check whether `df` is a PySpark DataFrame without importing PySpark.
 
     Warning:
         This method cannot be called on a Narwhals DataFrame/LazyFrame.
     """
-    _raise_if_narwhals_df_or_lf(df)
+    _warn_if_narwhals_df_or_lf(df)
     return bool(
         (pyspark_sql := get_pyspark_sql()) is not None
         and isinstance(df, pyspark_sql.DataFrame)
@@ -347,7 +369,7 @@ def is_pyspark_connect_dataframe(df: Any) -> TypeIs[PySparkConnectDataFrame]:
     Warning:
         This method cannot be called on a Narwhals DataFrame/LazyFrame.
     """
-    _raise_if_narwhals_df_or_lf(df)
+    _warn_if_narwhals_df_or_lf(df)
     if get_pyspark_connect() is not None:  # pragma: no cover
         try:
             from pyspark.sql.connect.dataframe import DataFrame
@@ -363,7 +385,7 @@ def is_sqlframe_dataframe(df: Any) -> TypeIs[SQLFrameDataFrame]:
     Warning:
         This method cannot be called on a Narwhals DataFrame/LazyFrame.
     """
-    _raise_if_narwhals_df_or_lf(df)
+    _warn_if_narwhals_df_or_lf(df)
     if get_sqlframe() is not None:
         from sqlframe.base.dataframe import BaseDataFrame
 
@@ -410,7 +432,7 @@ def is_pandas_like_dataframe(df: Any) -> bool:
     Warning:
         This method cannot be called on a Narwhals DataFrame/LazyFrame.
     """
-    _raise_if_narwhals_df_or_lf(df)
+    _warn_if_narwhals_df_or_lf(df)
     return is_pandas_dataframe(df) or is_modin_dataframe(df) or is_cudf_dataframe(df)
 
 
@@ -422,7 +444,7 @@ def is_pandas_like_series(ser: Any) -> bool:
     Warning:
         This method cannot be called on Narwhals Series.
     """
-    _raise_if_narwhals_series(ser)
+    _warn_if_narwhals_series(ser)
     return is_pandas_series(ser) or is_modin_series(ser) or is_cudf_series(ser)
 
 
@@ -436,15 +458,28 @@ def is_pandas_like_index(index: Any) -> bool:
     )  # pragma: no cover
 
 
+def is_pandas_like_dtype(obj: Any) -> TypeIs[PandasLikeDType]:
+    return bool(pd := get_pandas()) and isinstance(
+        obj, (pd.api.extensions.ExtensionDtype, get_numpy().dtype)
+    )
+
+
+def is_cudf_dtype(
+    obj: Any,
+) -> TypeIs[pd.api.extensions.ExtensionDtype]:  # pragma: no cover
+    return (
+        bool(pd := get_pandas())
+        and isinstance(obj, (pd.api.extensions.ExtensionDtype))
+        and hasattr(obj, "to_arrow")
+    )
+
+
 def is_into_series(native_series: Any | IntoSeriesT) -> TypeIs[IntoSeriesT]:
     """Check whether `native_series` can be converted to a Narwhals Series.
 
     Arguments:
         native_series: The object to check.
 
-    Returns:
-        `True` if `native_series` can be converted to a Narwhals Series, `False` otherwise.
-
     Examples:
         >>> import pandas as pd
         >>> import polars as pl
@@ -479,9 +514,6 @@ def is_into_dataframe(native_dataframe: Any | IntoDataFrameT) -> TypeIs[IntoData
     Arguments:
         native_dataframe: The object to check.
 
-    Returns:
-        `True` if `native_dataframe` can be converted to a Narwhals DataFrame, `False` otherwise.
-
     Examples:
         >>> import pandas as pd
         >>> import polars as pl
